pytrebuchet.drag_coefficient module
Drag coefficient calculations for a smooth sphere.
- pytrebuchet.drag_coefficient.clift_grace_weber(reynolds_number: float | ndarray[tuple[Any, ...], dtype[floating]]) float | ndarray[tuple[Any, ...], dtype[floating]]
Calculate the drag coefficient for a smooth sphere based on the Reynolds number.
ref: Clift, Grace, and Weber (Bubbles, Drops, and Particles, Academic Press, 1978)
- Parameters:
reynolds_number – Reynolds number of the sphere (dimensionless)
- Returns:
drag coefficient of the sphere (dimensionless)
- pytrebuchet.drag_coefficient.morrison(reynolds_number: float | ndarray[tuple[Any, ...], dtype[floating]]) float | ndarray[tuple[Any, ...], dtype[floating]]
Calculate the drag coefficient for a smooth sphere based on the Reynolds number.
Source: Data Correlation for Drag Coefficient for Sphere Faith A. Morrison, Department of Chemical Engineering Michigan Technological University
The correlation is valid for Reynolds numbers between 0.1 and 1e6. If the input Reynolds number is outside this range, a warning is issued.
- Parameters:
reynolds_number – Reynolds number of the sphere
- Returns:
drag coefficient of the sphere (dimensionless)
- pytrebuchet.drag_coefficient.calculate_reynolds_number(velocity: float, diameter: float, air_kinematic_viscosity: float) float
Calculate the Reynolds number for a (spherical) projectile.
- Parameters:
velocity – Velocity of the projectile (m/s)
diameter – Diameter of the projectile (m)
air_kinematic_viscosity – Kinematic viscosity of the air (m**2/s)
- Returns:
Reynolds number (dimensionless)